package scripts.migration.from4_4To4_5;

import scripts.migration.from4_4To4_5.*;


class Configuration{

    def config = [
        // Enter your modules to migrate
        //
        // name:Give the name of the module
        // migrationType: Set migrationType to STK if your module is based on STK templating, otherwise choose BASIC
        // templatingDependency: if your project references templates, paragraphs or dialogs form another modules, please list their names.
        //                       This will allows your template definition for example to point to the new
        //                       stk ID's.
        modules : [
//            "module1" :[
//                "name":"TheName1",
//                "migrationType":"STK",
//                "templatingDependency":["standard-templating-kit", "form"]
//            ],
//
//            "module2" :[
//                "name":"TheName2",
//                "migrationType":"BASIC"
//            ]
        ], // End of Modules config

        //Enter the site definition to migrate
        siteDef : [
//            "SiteDefPath1",
//            "SiteDefPath2"
        ],

        //Enter your websites to migrate (only STK Based)
        websites : [],

        // Magnolia Module to migrate
        // Any magnolia module that is not yet migrated has to be set to true
        migrateSTK : true,

        migrateDemoProject : true,

        magnoliaModulesToMigrate : [
            "commenting" : true,
            "public-user-registration": true,
            "form": true,
            "rssaggregator": true,
            "categorization": true,
            "forum": true,
            "shop": true,
            "dms": true,
            "resources": true
        ],

        // When you think that the config is ready set this flag to true.
        isConfigReady : true,

        // Intermediate save
        intermediateSave : false,

        // Deactivate node Listener (in order to avoid WARN log during migration process.)
        deactivateNodeListener : false
    ]
    ///// END CONFIG



    def isNodeListenerDeactivated(){
        return config.deactivateNodeListener
    }

    def isIntermediateSave(){
        return config.intermediateSave
    }


    def isConfigReady(){
        return config.isConfigReady
    }

    /**
     * Return a list of Magnolia modules to migrate.
     *
     */
    def getMagnoliaModulesToMigrate(){
        def modules = []
        config.magnoliaModulesToMigrate.entrySet().each{
            // check if the module is installed before adding it
            if(it.value == true){
                modules.add(it.key)
            }
        }
        return scripts.migration.from4_4To4_5.util.MigrationUtil.filterUninstalledModules(modules)
    }

    def getMagnoliaModulesIncludingSTK(){
        def list = getMagnoliaModulesToMigrate()
        if(migrateSTK()){
            list.add("standard-templating-kit")
        }
        return list
    }

    def getModules(type){
        def modules = []
        if(config.modules.size() > 0) {
            config.modules.entrySet().each{
                if(it.value.migrationType == type){
                    modules.add(it.value.name)
                }
            }
        }
        return scripts.migration.from4_4To4_5.util.MigrationUtil.filterUninstalledModules(modules)
    }

    /**
     * Return a list of custom modules that are based on STK.
     *
     */
    def getSTKBasedModules(){

        return getModules("STK")
    }

    /**
    * Return a list of custom modules that are based on the basic templating.
    *
    */
    def getBasicModules(){
        return getModules("BASIC")
    }

    /**
    * Return all the modules that have to perform the step "move", Magnolia and custom.
    * We have to handle a special:
    * we move only modules provinding templating and resources module do not provide such templates
    */
    def getAllModulestoMove(){
        def list = getModules("STK")+getModules("BASIC") + getMagnoliaModulesIncludingSTK()
        list = list - 'resources'
        return list;
    }

    /**
     * Return all the modules to migrate including the modules that are listed in the templatingDependency of the configuration.
     *
     * @return
     */
    def getModulesForIDCreation(){
        def list = getModules("STK")+getModules("BASIC")+getMagnoliaModulesIncludingSTK()
        if(config.modules.size() > 0) {
            config.modules.entrySet().each{
                if(it.value.templatingDependency!=null){
                    list = list + it.value.templatingDependency
                }
            }
        }
        return scripts.migration.from4_4To4_5.util.MigrationUtil.filterUninstalledModules(list.unique())
    }

    def getSiteDef(){
        return config.siteDef
    }

    def getWebsites(){
        def list = config.websites
        if(migrateDemoProject()){
            list.addAll(["/demo-project", "/demo-features"])
        }
        return scripts.migration.from4_4To4_5.util.MigrationUtil.filterNotExistingWebSites(list)
    }

    def migrateSTK(){
        return config.migrateSTK && scripts.migration.from4_4To4_5.util.MigrationUtil.moduleisInstalled("standard-templating-kit")
    }

    def migrateDemoProject(){
        return config.migrateDemoProject
    }

    def migrateForm(){
        return config.magnoliaModulesToMigrate.form && scripts.migration.from4_4To4_5.util.MigrationUtil.moduleisInstalled("form")
    }

    def migrateShop(){
        return config.magnoliaModulesToMigrate.shop && scripts.migration.from4_4To4_5.util.MigrationUtil.moduleisInstalled("shop")
    }

    def migrateResources(){
        return config.magnoliaModulesToMigrate.resources && scripts.migration.from4_4To4_5.util.MigrationUtil.moduleisInstalled("resources")
    }

    def migratePur(){
        return config.magnoliaModulesToMigrate.containsKey("public-user-registration") && scripts.migration.from4_4To4_5.util.MigrationUtil.moduleisInstalled("public-user-registration")
    }
}

