package scripts.migration.from4_4To4_5;

import scripts.migration.from4_4To4_5.Configuration;
import scripts.migration.from4_4To4_5.form.ExtraFormTasks;
import scripts.migration.from4_4To4_5.shop.ExtraShopTasks;
import scripts.migration.from4_4To4_5.general.ChangeReferencesToExtends;
import scripts.migration.from4_4To4_5.general.PostMigrationProcessing;
import scripts.migration.from4_4To4_5.general.RenameDialogsAndChangeInDialogsMap;
import scripts.migration.from4_4To4_5.general.TemplatesAndParagraphsToPagesAndComponents;
import scripts.migration.from4_4To4_5.general.PathChangesImpactingID;
import scripts.migration.from4_4To4_5.general.TransformExtends;
import scripts.migration.from4_4To4_5.general.UpdateMetaDataTemplateId;
import scripts.migration.from4_4To4_5.general.UpdateWebsitePrimaryType;
import scripts.migration.from4_4To4_5.general.TransformSimpleTemplates;
import scripts.migration.from4_4To4_5.pur.PURExtraTasks;
import scripts.migration.from4_4To4_5.resources.ResourcesExtraTasks;
import scripts.migration.from4_4To4_5.stk.RemoveTemplateToPrototypeRedundancy;
import scripts.migration.from4_4To4_5.stk.PreProcessSTKTemplates;
import scripts.migration.from4_4To4_5.stk.TransformSTKWebsites;
import scripts.migration.from4_4To4_5.stk.TransformSTKTemplates;
import scripts.migration.from4_4To4_5.stk.RemoveClassFromSTKSiteThemes;
import scripts.migration.from4_4To4_5.stk.ExtraSTKTasks;
import scripts.migration.from4_4To4_5.util.MigrationUtil;
import scripts.migration.from4_4To4_5.util.BuildDialogIdMap;
import scripts.migration.from4_4To4_5.util.ExportConfiguration;
import info.magnolia.cms.util.ObservationUtil;


config = new Configuration()


if(config.isConfigReady()){
    println "Config done, let's start to migrate"

    // Deactivate the node listener. Node Listener will be reactivate after the next restart of the application.
    if(config.isNodeListenerDeactivated()){
        ObservationUtil.dispose('config');
        ObservationUtil.dispose('website');
        println "Deactivate the node listener --> DONE"
    }

    /** STK Specific **/
    // Step that rename the folder paragraph of dialog into components
    if(config.migrateSTK()){
        pathChanges = new PathChangesImpactingID(ctx: ctx, workspace:'config')
        pathChanges.doChangePathes()
        MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
        println "STK: rename the folder paragraph of dialog into components --> DONE"
    }

    /** Step Move **/
    // This step is required by any modules, STK-based and Basic templating based.
    templatesAndParagraphsToPagesAndComponents = new TemplatesAndParagraphsToPagesAndComponents(ctx: ctx, workspace:'config')
    templatesAndParagraphsToPagesAndComponents.doModules(config.getAllModulestoMove())
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
    println "Modules: move templates and paragraphs to pages and components --> DONE"

    /** Step ID Creation **/
    // This step is required if you migrate a STK-based module, a basic templating-based module or a website.
    // So in any cases this step has to be executed.
    Map dialogsIdMap = new BuildDialogIdMap(ctx: ctx, workspace:'config').buildDialogIdMap(config.getModulesForIDCreation())
    Map pagesIdMap = MigrationUtil.buildPagesIdMapFromListOfModule(config.getModulesForIDCreation(), ctx)
    Map componentsIdMap = MigrationUtil.buildComponentsIdMapFromListOfModule(config.getModulesForIDCreation(), ctx)
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
    println "Modules: creation dialog, pages, components IdMap --> DONE"

    /** STK Specific **/
    // Step that rename dialogs of STK.
    if(config.migrateSTK()){
        renameDialogsAndChangeInDialogsMap = new RenameDialogsAndChangeInDialogsMap(ctx:ctx, dialogsIdMap:dialogsIdMap, workspace:'config')
        renameDialogsAndChangeInDialogsMap.doRenameDialogs(RenameDialogsAndChangeInDialogsMap.dialogsToRename())
        MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
        println "STK: rename dialog --> DONE"
    }

    /**** STK-based modules migration **/
    /** Step Pre-process STK-based template **/
    preProcessSTKTemplates = new PreProcessSTKTemplates(ctx: ctx, workspace:'config')
    preProcessSTKTemplates.doPreProcess(config.getSTKBasedModules(), config.getSiteDef())
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
    println "STK: pre-process STK template --> DONE"

    /** Step Transform STK-based templates **/
    transformSTKTemplates = new TransformSTKTemplates(ctx: ctx, workspace:'config', componentsIdMap:componentsIdMap, pagesIdMap:pagesIdMap, dialogsIdMap:dialogsIdMap)
    transformSTKTemplates.migrateModulesAndSiteDef(config.getSTKBasedModules(), config.getSiteDef())
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
    println "STK: transform STK template --> DONE"

    /** END STK-based modules **/


    /**** Basic templating-based module migration **/
    /** Step Transform simple **/
    transformTemplates = new TransformSimpleTemplates(ctx: ctx, workspace:'config', dialogsIdMap:dialogsIdMap)
    transformTemplates.doModules(config.getBasicModules())
    transformTemplates.log()
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
    println "BASIC: transform simple template --> DONE"
    /** END Basic templating-based module **/


    /**** STK-based website migration**/
    /** Step Update primary type **/
    updateWebsitePrimaryType = new UpdateWebsitePrimaryType(ctx: ctx, workspace:'website')
    updateWebsitePrimaryType.doWebsites(config.getWebsites())
    updateWebsitePrimaryType.log()
    MigrationUtil.save(ctx, 'website', config.isIntermediateSave())
    println "STK: website, update primary type --> DONE"

    /** Step Update template ID **/
    updateMetaData = new UpdateMetaDataTemplateId(ctx: ctx, workspace:'website', pagesIdMap:pagesIdMap, componentsIdMap:componentsIdMap)
    updateMetaData.doWebsites(config.getWebsites())
    updateMetaData.log()
    MigrationUtil.save(ctx, 'website', config.isIntermediateSave())
    println "STK: website, update meta data --> DONE"

    /** Step Transform STK-based websites **/
    transformSTKWebsites = new TransformSTKWebsites(ctx: ctx, workspace:'website')
    transformSTKWebsites.doWebsites(config.getWebsites());
    MigrationUtil.save(ctx, 'website', config.isIntermediateSave())
    println "STK: website, migrate site --> DONE"

    /** END STK-based website **/


    /**** HERE START MAGNOLIA MODULES MIGRATION **/
    /** STK Migration **/
    if(config.migrateSTK()){
        preProcessSTKTemplates.doPreProcessOfSTK()
        transformSTKTemplates.migrateSTK()

        remove = new RemoveTemplateToPrototypeRedundancy(ctx: ctx, workspace:'config')
        remove.doSTK()

        removeClassFromSTKSiteThemes = new RemoveClassFromSTKSiteThemes(ctx: ctx, workspace:'config')
        removeClassFromSTKSiteThemes.convertSites()
        removeClassFromSTKSiteThemes.log()

        transformExtends = new TransformExtends(ctx:ctx, workspace:'config')
        Map stkComparisonMap = transformExtends.prepareAndCreateSTKComparisonMap(ctx, 'config')
        transformExtends.executeComparison(stkComparisonMap, "standard-templating-kit/", "info.magnolia.module.templatingkit.messages")
        transformExtends.deletedI18nMessagesLog()
        MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
        println "STK: basic migration --> DONE"

        stkExtraTask = new ExtraSTKTasks(ctx:ctx, intermediatSave:config.isIntermediateSave())
        stkExtraTask.execute()
        println "STK: extra install migration task --> DONE"
    }


    /** Magnolia modules migration **/
    transformTemplates.doModules(config.getMagnoliaModulesToMigrate())
    transformTemplates.log()
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
    println "Module: transform module template --> DONE"

    if(config.migrateForm()){
        extraFormTask = new ExtraFormTasks(ctx: ctx, workspace:'config', componentsIdMap:componentsIdMap)
        extraFormTask.execute()
        MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
        println "Module Form: extra migration steps for the Form Module --> DONE"
    }

    if(config.migrateShop()){
        extraShopTask = new ExtraShopTasks(ctx: ctx, workspace:'config', componentsIdMap:componentsIdMap)
        extraShopTask.execute()
        MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
        println "Module Shop: extra migration steps for the Shop Module --> DONE"
    }


    if(config.migrateResources()){
        resourcesExtraTasks = new ResourcesExtraTasks(ctx: ctx)
        resourcesExtraTasks.execute()
        MigrationUtil.save(ctx, 'resources', config.isIntermediateSave())
        println "Module Resource: extra migration steps for the Resource Module --> DONE"
    }

    if(config.migratePur()){
        purExtraTasks = new PURExtraTasks(ctx: ctx, componentsIdMap:componentsIdMap)
        purExtraTasks.execute()
        purExtraTasks.updateGroupFiedsTemplateOnWebsites(config.getWebsites())
        purExtraTasks.updateOldProperties()
        MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
        MigrationUtil.save(ctx, 'website', config.isIntermediateSave())
        println "Module Pur: extra migration steps for the Pur Module --> DONE"
    }

    /** End Magnolia modules migration **/

    /***** Magnolia module post migration *****/
    changeReferencesToExtends = new ChangeReferencesToExtends(ctx:ctx, workspace:'config')
    changeReferencesToExtends.doModules(config.getMagnoliaModulesIncludingSTK())
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())

    postMigrationProcessing = new PostMigrationProcessing(ctx:ctx)
    postMigrationProcessing.executePropertyChanges()
    postMigrationProcessing.executeImportBootstraps()
    postMigrationProcessing.executeDeleteNodes()
    postMigrationProcessing.executeDeleteProperties()
    MigrationUtil.save(ctx, 'config', config.isIntermediateSave())
    MigrationUtil.save(ctx, 'website', config.isIntermediateSave())
    println "Migration: post migration process --> DONE"

    /** FINAL SAVE **/
    MigrationUtil.save(ctx, 'config', true)
    MigrationUtil.save(ctx, 'website', true)
    MigrationUtil.save(ctx, 'resources', true)

    println "Migration done !"
    }
else {
    println "Please update the configuration"
}