package scripts.migration.from4_4To4_5.general;

import java.util.Map;

import freemarker.template.utility.StringUtil;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.context.Context;

import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

class RenameDialogsAndChangeInDialogsMap {

  Context ctx
  HierarchyManager hm
  String workspace
  Map dialogsIdMap = [:]


  /**
   * Rename all dialogs defined in the passed map. The changed value is put into the dialogId map for all further processing.
   */
  def doRenameDialogs(Map dialogsToRename) {
      hm = ctx.getHierarchyManager(workspace);
      dialogsToRename.each{ String dialogPath, String newDialogName ->

          if(hm.isExist(dialogPath)){
              String oldDialogName = StringUtils.substringAfterLast(dialogPath, "/");

              MigrationUtil.renameAndMergeNodes(hm.getContent(dialogPath),newDialogName,false,true);

              String oldDialogId = dialogsIdMap.get(oldDialogName);
              String newDialogID = oldDialogId.replace(oldDialogName, newDialogName)
              dialogsIdMap.put(oldDialogName, newDialogID)
          }
      }
  }

  /**
   * All dialogs to be renamed. At this stage, its mainly Page-Header dialogs to Page-Intro dialogs.
   */
  def static Map dialogsToRename(){
      Map dialogsToRename = [
          "/modules/standard-templating-kit/dialogs/pages/faq/stkFAQHeader":"stkFAQIntro",
          "/modules/standard-templating-kit/dialogs/pages/form/stkFormHeader":"stkFormIntro",
          "/modules/standard-templating-kit/dialogs/pages/news/stkNewsHeader":"stkNewsIntro",
          "/modules/standard-templating-kit/dialogs/pages/event/stkEventHeader":"stkEventIntro",
          "/modules/standard-templating-kit/dialogs/pages/article/stkArticleHeader":"stkArticleIntro",
          "/modules/standard-templating-kit/dialogs/pages/siteMap/stkSiteMapHeader":"stkSiteMapIntro",
          "/modules/standard-templating-kit/dialogs/pages/glossary/stkGlossaryHeader":"stkGlossaryIntro",
          "/modules/standard-templating-kit/dialogs/pages/glossary/stkGlossaryTermHeader":"stkGlossaryTermIntro",
          "/modules/standard-templating-kit/dialogs/pages/glossary/stkGlossaryLetterHeader":"stkGlossaryLetterIntro",
          "/modules/standard-templating-kit/dialogs/pages/largeArticle/stkLargeArticleHeader":"stkLargeArticleIntro",
          "/modules/standard-templating-kit/dialogs/pages/imageGallery/stkImageGalleryHeader":"stkImageGalleryIntro",
          "/modules/standard-templating-kit/dialogs/pages/newsOverview/stkNewsOverviewHeader":"stkNewsOverviewIntro",
          "/modules/standard-templating-kit/dialogs/pages/eventsOverview/stkEventsOverviewHeader":"stkEventsOverviewIntro",
          "/modules/standard-templating-kit/dialogs/pages/section/stkSectionHeader":"stkSectionIntro",
          "/modules/standard-templating-kit/dialogs/pages/section/stkSectionBanner":"stkSectionHeader",
      ]
      return dialogsToRename
  }

}