package scripts.migration.from4_4To4_5.general;

import java.util.Collection;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.context.Context;
import info.magnolia.cms.util.ContentUtil;
import javax.jcr.Node;

import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

class TemplatesAndParagraphsToPagesAndComponents {
  Context ctx
  HierarchyManager hm
  String workspace

  /**
   * if module/moduleName has a template node:
   *   Merge this node (module/moduleName/template) to module/moduleName/template/page
   * if module/moduleName has a paragraphs node:
   *   Merge this node (module/moduleName/paragraphs) to module/moduleName/template/components
   */
  def moveTemplatesAndParagraphs(String modulePath){

    if(hm.isExist(modulePath)){
        Content moduleNode = hm.getContent(modulePath)

        if(moduleNode.hasContent("templates")){
            // Check if template has still a pages node
            if(moduleNode.hasContent("templates/pages")) {
                // Iterate template chields and move then to pages
                ContentUtil.getAllChildren(moduleNode.getContent("templates")).each{ Content child ->
                    if(!child.getName().equals("pages") && !child.getName().equals("components")) {
                        MigrationUtil.moveAndMergeNodes(child,moduleNode.getHandle()+"/templates/pages",false,true)
                    }
                }
            } else {
                // As destination path can not be a  descendant of source path we have to rename the templates folder first.
                Content pagesNode = MigrationUtil.renameAndMergeNodes(moduleNode.getContent("templates"), "pages", false, true)
                // Then recreate the Node
                MigrationUtil.getOrCreateContent(moduleNode, "templates", ItemType.CONTENT)
                //Moving the paragrpahs node into the templates node and rename to components
                MigrationUtil.renameAndMergeNodes(pagesNode, "templates/pages", false, true)
            }
          }

        if(moduleNode.hasContent("paragraphs")){
            MigrationUtil.getOrCreateContent(moduleNode, "templates", ItemType.CONTENT)
            //Moving the paragrpahs node into the templates node and rename to components
            MigrationUtil.renameAndMergeNodes(moduleNode.getContent("paragraphs"), "templates/components", false, true)
          }
    }
  }

  /**
   * Call this def to move the templates and paragraphs of your module to the new structure
   */
  def doSTK(String moduleName){
    doModule("standard-templating-kit")
  }

  /**
   * Call this def to move the templates and paragraphs of your module to the new structure
   */
  def doModule(String moduleName){
    hm = ctx.getHierarchyManager(workspace)
    String modulePath = "/modules/"+moduleName
    if(hm.isExist(modulePath)){
      moveTemplatesAndParagraphs(modulePath)
    }
  }

  /**
   * Call this def to move the templates and paragraphs of your modules
   */
  def doModules(Collection moduleNames){
    moduleNames.each{ String moduleName -> doModule(moduleName) }
  }

}
