package scripts.migration.from4_4To4_5.general;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.core.MetaData;
import info.magnolia.cms.util.ContentUtil
import info.magnolia.context.Context;

import org.apache.commons.lang.StringUtils;

/**
 * This class contains several methods updating the template of content found in websites, workspace.
 * Before executing one of these medthod you have to create a map containing the new template id.
 * See MigrationUtil#buildTemplateIdMap for further details on the creation of a map containing the new template ids.
 *
 */
class UpdateMetaDataTemplateId {

  Context ctx
  HierarchyManager hm
  String workspace

  // this map could contain any templates found under yourmodule/template
  Map allTemplatesIdMap = [:]
  // this map must contain only the template under yourmodule:templates/components
  Map componentsIdMap = [:]
  // this map must contain only the template under yourmodule:templates/pages
  Map pagesIdMap = [:]

  //log
  Collection areaLog = []

  def updateTemplate(Content content, Map templateIdMap){
    def template =  content.metaData.template
    if(StringUtils.isNotEmpty(template) && templateIdMap.containsKey(template)){
      def newTemplate = templateIdMap.get(template)
      content.getMetaData().setTemplate(newTemplate)
    }
  }

  def visitor = { Content content ->
    if(content.getNodeTypeName() == ItemType.PAGE.toString()){
      updateTemplate(content, pagesIdMap)
    }
    else if(content.getNodeTypeName() == ItemType.COMPONENT.toString()){
      updateTemplate(content, componentsIdMap)
    }
    else if(content.getNodeTypeName() == ItemType.AREA.toString()){
      // areas shouldnt have a template
      def template =  content.metaData.template
      if(StringUtils.isNotEmpty(template)){
        areaLog.add("Area "+ content.getName() +" has a template... This is not correct !! Verify this node --> "+content.getHandle())
      }
    }
    // for all the other types - this should be the case of modules provinding no templating
    else{
      updateTemplate(content, allTemplatesIdMap)
    }
  } as ContentUtil.Visitor

  def updateFooter(Content siteRoot) {
    Content footer = siteRoot.getContent("footer")
    footer.getMetaData().removeProperty(MetaData.TEMPLATE)
  }

  /**
   * Update the template id of the websites found in the list.
   *
   */
  def doWebsites(Collection websites) {
    hm = ctx.getHierarchyManager(workspace);
    websites.each{String sitePath ->
      def root = hm.getContent(sitePath);
      ContentUtil.visit(root, visitor)
    }
  }

  /**
   * Update the template id of content found in the workspace
   *
   */
  def doWorkspace(String doWorkspace) {
    println "** Start migration of metaData of all defined workspaces **"
    hm = ctx.getHierarchyManager(doWorkspace);
    Content root = hm.getRoot()
    ContentUtil.visit(root,visitor)
  }

  def log(){
    println "**** Areas with template"
    println areaLog
  }
}