package scripts.migration.from4_4To4_5.general;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.context.Context;

import org.apache.commons.lang.StringUtils;
import scripts.migration.from4_4To4_5.util.MigrationUtil;

class UpdateWebsitePrimaryType {

  Context ctx
  HierarchyManager hm
  String workspace
  Content tmpChild

  Collection singleAreas = ["stage", "opener"]

  //for log
  Collection singleAreaUpdated = []

  /**
   * Return true if an area is type 'single'. This test can be used on content before the step "update primary type".
   *
   * @param content
   * @return
   */
  def isAreaSingle(Content content){
    return (content.getName() in singleAreas) && (content.getNodeTypeName() == ItemType.CONTENTNODE.toString())
  }

  /**
   * Change the type of a node.
   * In order to reduce the amount of data handled by the method ContentUtil#changeNodeType,
   * the node is temporary isolated in a temp node.
   *
   */
  def changeNodeType(Content node, ItemType type, Boolean isSingle){
    def nodeName = node.getName()
    def destBack = node.getHandle()
    def parentPath = node.getParent().getHandle()

    //move children to tmpChildren node
    ContentUtil.getAllChildren(node).each{ Content child ->
      MigrationUtil.moveAndMergeNodes(child, tmpChild.getHandle(), false, true)
    }

    //change node type
    // TODO for performance matter, changeNodeType should create a file in memory and not in the file system.
    ContentUtil.changeNodeType(node,type, false)
    node = hm.getContent(destBack)

    // move children back
    ContentUtil.getAllChildren(tmpChild).each{ Content child ->
      MigrationUtil.moveAndMergeNodes(child, destBack, false, true)
    }

    if(isSingle){
     MigrationUtil.renameAndMergeNodes(hm.getContent(destBack),"component",false, true);
      hm.createContent(parentPath,nodeName, ItemType.AREA.toString())
      MigrationUtil.moveAndMergeNodes(hm.getContent(parentPath+"/component"),parentPath+"/"+nodeName, false, true)
      node = hm.getContent(parentPath+"/"+nodeName+"/component")
    }

    return node;
  }

  /**
   * Return the new type (page, component or area).
   * Content -> Page
   * Content node -> Area (if content has no template)
   * Content node -> Component (if content has a template)
   *
   */
  def getNewType(Content content){
    def oldType = content.getNodeTypeName()
    if(oldType == ItemType.CONTENT.toString())
      return ItemType.PAGE
    else{
      if(StringUtils.isEmpty(content.getTemplate())){
        return ItemType.AREA
      }
      else{
        return ItemType.COMPONENT
      }
    }
  }

  def updateTree(Content content){
    if(content.getNodeTypeName() in [ItemType.PAGE.toString(),ItemType.AREA.toString(),ItemType.COMPONENT.toString(),ItemType.NT_RESOURCE.toString()] ) {
        return;
    }

    if(isAreaSingle(content)){
      content = changeNodeType(content, ItemType.COMPONENT, true)
      singleAreaUpdated.add("Single area "+content.getHandle()+" updated.")
    }
    else{
      def  newType = getNewType(content)
      content = changeNodeType(content, newType, false)
    }
    ContentUtil.getAllChildren(content).each{ Content child ->
      updateTree(child)
    }
  }

  /**
   * Call this method to run the complete update of your websites.
   *
   */
  def doWebsites(Collection websites){
    hm = ctx.getHierarchyManager(workspace);
    tmpChild = hm.createContent("/","tmpChildren","mgnl:content")
    websites.each{ String sitePath ->
      def root = hm.getContent(sitePath)
      updateTree(root)
    }
    ContentUtil.deleteAndRemoveEmptyParents(tmpChild)
  }

  def log()
  {
    println " *** Single Area updated "
    println singleAreaUpdated
  }
}
