package scripts.migration.from4_4To4_5.pur;

import java.util.Collection;

import scripts.migration.from4_4To4_5.util.MigrationUtil;
import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.context.Context;

/**
 * The goal of this step is to do the extra tasks of PUR.
 *
 */
class PURExtraTasks {

  Context ctx
  HierarchyManager hm
  Map componentsIdMap

  def groupFieldsName = "groupFields"
  def groupFieldsID = "standard-templating-kit:components/pur/groupFields"

  def createGroupFields(){
    if(MigrationUtil.moduleisInstalled("standard-templating-kit")){
      def stkPurComponents = hm.getContent("/modules/standard-templating-kit/templates/components/pur")
      def groupFields = MigrationUtil.getOrCreateContent(stkPurComponents, groupFieldsName, ItemType.CONTENTNODE)
      groupFields.setNodeData("extends", "/modules/form/templates/components/formGroupFields")
      def areas = MigrationUtil.createAreasNode(groupFields)
      def availableComponents = [
        "formEdit",
        "formPassword",
        "stkPURNotAuthSubmit"
      ]
      String templateScript = MigrationUtil.getGenericTemplateScript("list")
      def fields = MigrationUtil.createArea(areas, "fields", templateScript, "list", true, null, null, availableComponents, null, componentsIdMap)
      fields.getContent("availableComponents").setNodeData("extends","override")
      componentsIdMap.put(groupFieldsName, groupFieldsID)
    }
  }

  def transformPURRegistrationForm(){
	if(hm.isExist('/modules/standard-templating-kit/templates/components/pur/stkPURRegistrationForm')){
	    Content registrationForm = hm.getContent('/modules/standard-templating-kit/templates/components/pur/stkPURRegistrationForm')
	    if(!registrationForm.hasContent("areas")){
	      registrationForm.delete("availableComponents")
	      Content areas = MigrationUtil.createAreasNode(registrationForm)
	      def availableComponents = [
	        groupFieldsName
	      ]
	      String templateScript = MigrationUtil.getGenericTemplateScript("list")
	      MigrationUtil.createArea(areas, "fieldsets", templateScript, "list", true, null, null, availableComponents, null, componentsIdMap)
	    }
	}
  }

  def transformPURPasswordForm(){
	if(hm.isExist('/modules/standard-templating-kit/templates/components/pur/stkPURPasswordForm')){
	    Content registrationForm = hm.getContent('/modules/standard-templating-kit/templates/components/pur/stkPURPasswordForm')
	    if(!registrationForm.hasContent("areas")){
	      registrationForm.delete("availableComponents")
	      Content areas = MigrationUtil.createAreasNode(registrationForm)
	      def availableComponents = [groupFieldsName]
	      String templateScript = MigrationUtil.getGenericTemplateScript("list")
	      MigrationUtil.createArea(areas, "fieldsets", templateScript, "list", true, null, null, availableComponents, null, componentsIdMap)
	    }
	}
  }

  def transformPURUpdateForm(){
	if(hm.isExist('/modules/standard-templating-kit/templates/components/pur/stkPURUpdateForm')){
	    Content registrationForm = hm.getContent('/modules/standard-templating-kit/templates/components/pur/stkPURUpdateForm')
	    if(!registrationForm.hasContent("areas")){
	      registrationForm.delete("availableComponents")
	      Content areas = MigrationUtil.createAreasNode(registrationForm)
	      def availableComponents = [groupFieldsName]
	      String templateScript = MigrationUtil.getGenericTemplateScript("list")
	      MigrationUtil.createArea(areas, "fieldsets", templateScript, "list", true, null, null, availableComponents, null, componentsIdMap)
	    }
	}
  }

  /**
   * This method update the content of websites.
   * As a new component "pur:component/groupFields" is created and available from the components stkPURRegistrationForm, stkPURPasswordForm and stkPURUpdateForm.
   * The node with these template have to update their fieldset template.
   *
   */
  def updateGroupFiedsTemplate(def root){
    ContentUtil.collectAllChildren(root, new Content.ContentFilter() {
          public boolean accept(Content content) {
            return (
            content.getTemplate().equals("standard-templating-kit:components/pur/stkPURRegistrationForm")
            || content.getTemplate().equals("standard-templating-kit:components/pur/stkPURPasswordForm")
            || content.getTemplate().equals("standard-templating-kit:components/pur/stkPURUpdateForm"));
          }
        }).each{
          if(it.hasContent("fieldsets")){
            def fieldset = it.getContent("fieldsets")
            ContentUtil.getAllChildren(fieldset).each{ Content c ->
              if(c.getTemplate().equals("form:components/formGroupFields")){
                c.getMetaData().setTemplate(groupFieldsID)
              }
            }
          }
        }
  }

  def updateTemplateScripts(){
    def mapOfChanges = [
          "userLogin":"publicuserregistration/components/user-login.ftl",
          "profileUpdate":"publicuserregistration/components/user-profile.ftl",
          "userRegistration":"publicuserregistration/components/user-registration.ftl",
          "passwordReminder":"publicuserregistration/components/password-reminder-username.ftl"
        ]

    Content components = hm.getContent('/modules/public-user-registration/templates/components')
    def componentsList = []
    MigrationUtil.visitTemplates(components, componentsList)
    componentsList.each{
      if(mapOfChanges[it.getName()]!= null){
        it.setNodeData("templateScript", mapOfChanges[it.getName()])
      }
    }
  }

  /**
   * Next to a modification in Form, some properties are obsolete and are not updated by the version handler of Form.
   * The update of these properties is done during the migration process.
   */
  def updateOldProperties(){
    hm = ctx.getHierarchyManager('website');
    def nodes = [
      "/demo-project/members-area/registration/main/content/0",
      "/demo-project/members-area/password-reminder/main/content/0",
      "/demo-features/modules/public-user-registration/PURRegistration/main/content/0",
      "/demo-features/modules/public-user-registration/PURUpdate/main/content/0"
    ]

    nodes.each{ String path ->
      if(MigrationUtil.webSiteExists(path)){
        def content = hm.getContent(path)
        if(content.hasNodeData("confirmMailBody")){
          def newName= "confirmContentType"+content.getNodeData("confirmContentType").getString()
          MigrationUtil.renameProperty(content.getNodeData("confirmMailBody"),newName)
        }
      }
    }
  }

  def execute(){
    if(MigrationUtil.moduleisInstalled("public-user-registration")){
      hm = ctx.getHierarchyManager('config')
      updateTemplateScripts()
      createGroupFields()
      transformPURRegistrationForm()
      transformPURPasswordForm()
      transformPURUpdateForm()
    }
  }

  def updateGroupFiedsTemplateOnWebsites(Collection websites) {
    if(MigrationUtil.moduleisInstalled("public-user-registration")){
      hm = ctx.getHierarchyManager('website')
      websites.each{ String sitePath ->
        def root = hm.getContent(sitePath)
        updateGroupFiedsTemplate(root)
      }
    }
  }
}