package scripts.migration.from4_4To4_5.stk;

import info.magnolia.cms.core.HierarchyManager;
import scripts.migration.from4_4To4_5.util.MigrationUtil;
import org.apache.commons.lang.StringUtils;
import javax.jcr.Property
import javax.jcr.Session;
import javax.jcr.Node;
import info.magnolia.cms.core.NodeData;
import info.magnolia.context.Context;

import info.magnolia.jcr.util.NodeUtil;
/**
 * The goal of this step is to do the extra tasks of STK.
 * All the tasks that are not directly linked to the main migration.
 * An extra task could remove inconsistency, like for example if a dialog is not properly named and all other stuff like that.
 *
 * The goal of this task is also to reduce the complexity of the other steps. We often remarked that some special cases have to be handled in the migration of STK.
 * This could be to best place for those kind of things.
 *
 * This step has to be executed at the very end of the process.
 *
 */
class ExtraSTKTasks {

    HierarchyManager hm
    Context ctx
    boolean intermediatSave

    def execute(){
        hm = ctx.getHierarchyManager('config');

        //Add templateScript/title property for stkHome and stkSection
        def stkHome = hm.getContent('/modules/standard-templating-kit/templates/pages/stkHome/areas/main/areas/content');
        stkHome.createNodeData('templateScript','/templating-kit/pages/section/contentArea.ftl');
        stkHome.createNodeData('title','templates.stkHome.areas.main.content.title');
        def stkSection = hm.getContent('/modules/standard-templating-kit/templates/pages/stkSection/areas/main/areas/content');
        stkSection.createNodeData('templateScript','/templating-kit/pages/section/contentArea.ftl');
        stkSection.createNodeData('title','templates.stkSection.areas.main.content.title');

        hm.delete("/modules/standard-templating-kit/dialogs/generic/teasers/tabLinkList");

        //Move Commenting references (Previously in the version handler)
        if(hm.isExist("/modules/commenting/templates/components/comments")) {
            def stkComments = hm.getContent('/modules/commenting/templates/components/comments');
            if(stkComments.hasNodeData('templateScript'))  {
                def prop = stkComments.getNodeData('templateScript');
                prop.setValue('/info/magnolia/module/commenting/frontend/comments.ftl');
            } else {
                stkComments.createNodeData('templateScript','/info/magnolia/module/commenting/frontend/comments.ftl');
            }
        }

        Map newParameters = [
                    "headingLevel" : "h3"
                ]
        def stkTeaserLinkList = hm.getContent('/modules/standard-templating-kit/templates/components/teasers/stkTeaser/areas/linkList');
        MigrationUtil.createParameters(stkTeaserLinkList,newParameters);

        Session website = ctx.getJCRSession("website");
        NodeUtil.getNodes(website.rootNode, "mgnl:page").each{processLinkList(it)}
        // SAVE
        if(intermediatSave) {
            website.save()
        }

        Session config = ctx.getJCRSession("config");
        if (config.nodeExists("/modules/standard-templating-kit/config/site/templates/prototype/areas")) {
            Node prototypeArea = config.getNode("/modules/standard-templating-kit/config/site/templates/prototype/areas");

            if (prototypeArea.hasNode("sectionHeader")) {
                Node sectionHeader = prototypeArea.getNode("sectionHeader")
                sectionHeader.setProperty("editable", Boolean.FALSE);
            }
            if (prototypeArea.hasNode("footer")) {
                Node footer = prototypeArea.getNode("footer")
                footer.setProperty("editable", Boolean.FALSE);
            }
            if (prototypeArea.hasNode("branding/areas/metaNavigation")) {
                Node metaNavigation = prototypeArea.getNode("branding/areas/metaNavigation")
                metaNavigation.setProperty("editable", Boolean.FALSE);
            }
        }
        // SAVE
        if(intermediatSave) {
            config.save()
        }
        //TODO: footer component need to be removed.
    }

    /**
     * move linkListTitle values and remove hasLinkArea properties.
     */
    def processLinkList(Node node){
        if(node.hasProperty("hasLinkList")){
            node.getProperty("hasLinkList").remove()
        }
        if(node.hasProperty("linkListTitle")){
            Property property = node.getProperty("linkListTitle")
            Node area;
            if(!node.hasNode("linkList"))
                area = node.addNode("linkList", "mgnl:area")
            else
                area = node.getNode("linkList")
            area.setProperty("title", property.getString())
            property.remove()
        }
        node.getNodes().each{processLinkList(it)}
    }
}
