package scripts.migration.from4_4To4_5.stk;

import info.magnolia.cms.core.Content
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.MgnlNodeType;
import info.magnolia.cms.core.NodeData
import info.magnolia.context.Context;


/**
 * Since STK 2.0, the ThemeReference is only a POJO keeping the name of the referenced theme.
 * Hence we no longer need the 'class' property on it.
 */
class RemoveClassFromSTKSiteThemes {
    Context ctx
    String workspace

    //track updated and unchanged sites for logging
    def updatedSites = []
    def unchangedSites = []

    def convertSites(){
        resetLog()
        HierarchyManager hm = ctx.getHierarchyManager(workspace)
        convertSTKSite(hm)
        convertETKSites(hm)
    }

    def convertSTKSite(hm){
        if(hm.isExist('/modules/standard-templating-kit/config/site')){
            def stkNode = hm.getContent('/modules/standard-templating-kit/config/site')
            updateSite(stkNode)
        }
    }

    def convertETKSites(hm){
        if(hm.isExist('/modules/extended-templating-kit/config/sites')){
            def etkSites = hm.getContent('/modules/extended-templating-kit/config/sites')
            etkSites.getChildren(MgnlNodeType.NT_CONTENTNODE).each{updateSite(it)}
        }
    }

    def updateSite(siteNode){
        def isUpdated = false
        def themeNodeName = 'theme'
        if (siteNode.hasContent(themeNodeName)) {
            Content themeNode = siteNode.getContent(themeNodeName)
            NodeData themeClass = themeNode.getNodeData('class')
            if (themeClass.isExist()) {
                themeClass.delete()
                isUpdated = true;
            }
        }
        isUpdated ?  updatedSites.add(siteNode.getHandle()) : unchangedSites.add(siteNode.getHandle())
    }

    def resetLog(){
        updatedSites = []
        unchangedSites = []
    }

    def log(){
        if(updatedSites == null) {
            return;
        }
        println " *** "
        println " *** Start log summary of TransformSTKSitesThemesToString script"
        println " *** "
        println " ** Updated Sites "
        println      updatedSites
        println " ** Unchanged Sites "
        println      unchangedSites
        println " *** End log summary of TransformSTKSitesThemesToString script"
        println " *** "
    }
}
