package scripts.migration.from4_4To4_5.stk;

import java.util.Collection;
import java.util.Map;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.util.ContentUtil
import info.magnolia.context.Context;

import org.apache.commons.lang.StringUtils;

import scripts.migration.from4_4To4_5.util.MigrationUtil;

class RemoveTemplateToPrototypeRedundancy {

  Context ctx
  HierarchyManager hm
  String workspace

  Map protoPropertiesMap = [:]
  Content currentPageTemplate
  Collection emptyNodesPaths = []

  /**
   * Get the path relative to the template
   */
  def static private getRelativePath(NodeData property, String templateName){
    return StringUtils.substringAfter(property.getHandle(), templateName+"/")
  }

  /**
   * Fill a map representing the protoytpe structure
   *
   */
  def private buildProtoMap(Content prototype){
    ContentUtil.visit(prototype, prototypeVisitor)
  }

  def private prototypeVisitor = { Content node ->
    node.getNodeDataCollection().each{ NodeData property ->
      protoPropertiesMap.putAt(getRelativePath(property,"prototype"), property.getString())
    }
  } as ContentUtil.Visitor

  /**
   * True if the property is already defined in the prototype with the same value
   */
  def private isRedundant(NodeData property, String templateName){
    def value = property.getString()
    def propKey = getRelativePath(property,templateName)
    return protoPropertiesMap[propKey] == value
  }

  /**
   * Compare the page template configuration with the prototype configuration and remove the redundancies
   */
  def private compareWithProto(Content pageTemplate){
    currentPageTemplate = pageTemplate
    ContentUtil.visit(pageTemplate, pageTemplateVisitor)
  }

  def private pageTemplateVisitor = {
    Content node ->
    if(node.getNodeDataCollection().size()>0){
      node.getNodeDataCollection().each{ NodeData property ->
        if(isRedundant(property,currentPageTemplate.getName())){
          //FIXME remove this condition when SCRUM-503 is solved
          if(property.getName()!="class"){
            node.deleteNodeData(property.getName())
          }
        }
      }
      //Node is empty and can there for be deleted.
      if(node.getChildren().size()==0 && node.getNodeDataCollection().size() == 0)  {
          emptyNodesPaths.add(node.getHandle())
      }
    }
  } as ContentUtil.Visitor

  /**
   * Deletes all empty nodes collected during all template visits.
   */
  def private deleteEmptyNodes(){
      emptyNodesPaths.each{ String emptyNodePath ->
        if (hm.isExist(emptyNodePath)) {
          hm.delete(emptyNodePath)
        }
      }
  }

  /**
   * Execute the remove redundancies steps on a module.
   */
  def doOnModule(String moduleName, String siteDefintionPath){
    Content pages = hm.getContent('/modules/'+moduleName+'/templates/pages')
    Collection listOfPagesTemplates = []
    MigrationUtil.visitTemplates(pages, listOfPagesTemplates)

    String prototypePath = siteDefintionPath +"/templates/prototype"
    Content prototype = hm.getContent(prototypePath)
    buildProtoMap(prototype)
    listOfPagesTemplates.each{ Content template -> compareWithProto(template) }
    deleteEmptyNodes()
  }



  /**
   * Remove redundancies between all templates located within the passed module in comparison to all passed site definitions.
   * @param moduleName The module name of where all templates are located to compare.
   * @param siteDefinitionPaths All the site definitions which should be compared with the modules templates.
   * This can occur in projects where different definitions share the same templates.
   */
  def doOnModules(String moduleName, Collection siteDefinitionPaths) {
      siteDefinitionPaths.each{ String siteDefintionPath ->
          doOnModule(moduleName, siteDefintionPath)
      }
  }


  /**
   * Execute the remove redundancies steps on STK.
   */
  def doSTK(){
    hm = ctx.getHierarchyManager(workspace)
    String stkSiteDefinitionPath = "/modules/standard-templating-kit/config/site"
    String etkSiteDefinitionPath = "/modules/extended-templating-kit/config/sites/default"

    //ETK && EE is installed
    if(hm.isExist(etkSiteDefinitionPath)){
        doOnModules("standard-templating-kit", [etkSiteDefinitionPath])
    }
    // Only CE is installed (no ETK or EE, ons STK)
    else {
        doOnModules("standard-templating-kit", [stkSiteDefinitionPath])
    }
  }
}
