package scripts.migration.from4_4To4_5.util;

import info.magnolia.cms.core.Content;
import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.core.ItemType;
import info.magnolia.cms.core.NodeData;
import info.magnolia.cms.util.ContentUtil;
import info.magnolia.context.Context;

import org.apache.commons.lang.StringUtils;

class BuildDialogIdMap{

  Context ctx
  HierarchyManager hm
  String workspace

  // will be populate like this: [dialog-name, path-to-dialog]
  // paths are going to look like this: <module-name:path-to-dialog-within-dialog-folder>
  Map dialogPathMap = [:]
  String modulePath = "/modules/"
  String moduleName = ""
  String dialogPath = ""
  int count = 0
  int countReplace = 0

  def setModuleName (String moduleName) {
    this.moduleName = moduleName
    modulePath += moduleName
    dialogPath = modulePath + "/dialogs"
  }

  def initParam () {
    modulePath = "/modules/"+moduleName
    dialogPath = modulePath + "/dialogs"
  }

  def composeNewPath (String pathInModule, String dialogName) {
    pathInModule = StringUtils.removeStart(pathInModule + "/" + dialogName, "/")
    return moduleName + ":" + pathInModule
  }

  def populateDialogPathMap (Content content, String folder) {
    ContentUtil.getAllChildren(content).each {
      if (it.getItemType() == ItemType.CONTENTNODE) {
        dialogPathMap.put(it.getName(), composeNewPath(folder, it.getName()))
        count++
      }
      else if (it.getItemType() == ItemType.CONTENT) {
        def subPath = folder + "/" + it.getName()
        populateDialogPathMap(it, subPath)
      }
    }
  }

  def replaceDialogPath(Content content) {
    ContentUtil.collectAllChildren(content).each{ Content moduleChild ->
      moduleChild.getNodeDataCollection('dialog').each { NodeData configWithDialog ->
        def parentNode = configWithDialog.parent
        parentNode.setNodeData("dialog", dialogPathMap.get(configWithDialog.value.string))
        countReplace++
      }
    }
  }

  def initDialogPathMap(){
    initParam()
    hm = ctx.getHierarchyManager(workspace)
    if (hm.isExist(dialogPath)) {
      def dialogContent = hm.getContent(dialogPath)
      populateDialogPathMap(dialogContent, "")
    }
    return dialogPathMap
  }

  def buildDialogIdMap(Collection moduleNameList){
    def map = [:]
    moduleNameList.each{ String name ->
      moduleName = name
      map.putAll(initDialogPathMap())
    }
    return map
  }

  //FIXME is it still used?
  def doDialogMigration() {
    // setup
    setModuleName("standard-templating-kit")
    hm = ctx.getHierarchyManager(workspace)
    // populate referencial map
    def dialogContent = hm.getContent(dialogPath)
    populateDialogPathMap(dialogContent, "")
    // replace dialogs with new way of referencing them
    def moduleContent = hm.getContent(modulePath)
    replaceDialogPath(moduleContent)
    return "found $count dialogs and $countReplace replacements - done!"
  }
}


